from fastapi_users.authentication import CookieAuthentication
from .config import getConfig

config = getConfig()
cookieAuthentication = CookieAuthentication(
    secret=config.secret, 
    lifetime_seconds=3600, 
    cookie_secure=config.cookie_secure,
)
authBackends = [cookieAuthentication]



import factory
from freenit.models.user import UserModel


class UserFactory(factory.Factory):
    class Meta:
        model = UserModel

    id = factory.Faker('uuid4')
    is_active = True
    email = factory.Faker('email')
    hashed_password = 'Sekrit'



import pytest
from freenit.models.user import UserModel


class TestUser():
    @pytest.mark.asyncio
    async def test_get_all(self, db_setup, user_factory):
        user = user_factory()
        await user.save()

    @pytest.mark.asyncio
    async def test_get_new(self, db_setup, user_factory):
        count = await UserModel.objects.count()
        assert count == 0



import os
import pytest
from pytest_factoryboy import register
from .factories import UserFactory


register(UserFactory)


@pytest.fixture
def db_setup():
    from freenit.app import app as fastapp
    from alembic.config import Config
    from alembic import command

    alembic_cfg = Config("alembic.ini")
    command.upgrade(alembic_cfg, "head")

    yield fastapp

    current_path = os.path.dirname(__file__)
    os.remove(f'{current_path}/../test.sqlite')



from logging.config import fileConfig

from alembic import context
from freenit.config import getConfig

config = getConfig()
fileConfig(context.config.config_file_name)


def run_migrations_offline():
    context.configure(
        url=config.dburl,
        target_metadata=config.metadata,
        literal_binds=True,
        dialect_opts={"paramstyle": "named"},
        user_module_prefix="sa.",
    )
    with context.begin_transaction():
        context.run_migrations()


def run_migrations_online():
    connectable = config.engine
    with connectable.connect() as connection:
        context.configure(
            connection=connection,
            target_metadata=config.metadata,
            user_module_prefix="sa.",
        )
        with context.begin_transaction():
            context.run_migrations()



from freenit.app import app
from freenit.config import getConfig


config = getConfig()
print()
print(f"    http://{config.hostname}:{config.port}/api/v1/docs")
print(f"    http://{config.hostname}:{config.port}/api/v1/redoc")
print()



"""initial

Revision ID: 14eb8b6e464c
Revises: 
Create Date: 2021-07-18 12:28:52.118800

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "14eb8b6e464c"
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "users",
        sa.Column("id", sa.CHAR(36), nullable=False),
        sa.Column("email", sa.String(length=255), nullable=False),
        sa.Column("hashed_password", sa.String(length=255), nullable=False),
        sa.Column("is_active", sa.Boolean(), nullable=False),
        sa.Column("is_superuser", sa.Boolean(), nullable=False),
        sa.Column("is_verified", sa.Boolean(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_users_email"), "users", ["email"], unique=True)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f("ix_users_email"), table_name="users")
    op.drop_table("users")
    # ### end Alembic commands ###



import os


try:
    from .local_config import DevConfig
except ImportError:
    from .base_config import DevConfig

try:
    from .local_config import TestConfig
except ImportError:
    from .base_config import TestConfig

try:
    from .local_config import ProdConfig
except ImportError:
    from .base_config import ProdConfig


configs = {}
configs[DevConfig.envname] = DevConfig()
configs[TestConfig.envname] = TestConfig()
configs[ProdConfig.envname] = ProdConfig()


def getConfig():
    config_name = os.getenv("FREENIT_ENV", "prod")
    return configs.get(config_name, configs["prod"])



#!/usr/bin/env python

import pathlib

from setuptools import find_packages, setup

PROJECT_ROOT = pathlib.Path(__file__).parent
README = (PROJECT_ROOT / 'README.md').read_text()

setup(
    name='NAME',
    version='0.0.1',
    description='REST API framework based on FastAPI',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/freenit-framework/backend',
    author='Goran Mekić',
    author_email='meka@tilda.center',
    license='BSD',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP',
        'Environment :: Web Environment',
        'Framework :: FastAPI',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    keywords=[
        'REST',
        'openapi',
        'swagger',
        'fastapi',
    ],
    packages=find_packages(exclude=['tests*']),
    python_requires='>=3.8',
    install_requires=['freenit'],
    extras_require={
        'test': [
            'pytest-asyncio',
            'pytest-factoryboy',
        ],
    },
)



from typing import Optional

import ormar
import pydantic

from ..config import getConfig

config = getConfig()


class AllOptional(pydantic.main.ModelMetaclass):
    def __new__(self, name, bases, namespaces, **kwargs):
        annotations = namespaces.get("__annotations__", {})
        for base in bases:
            annotations = {**annotations, **base.__annotations__}
        for field in annotations:
            if not field.startswith("__"):
                annotations[field] = Optional[annotations[field]]
        namespaces["__annotations__"] = annotations
        return super().__new__(self, name, bases, namespaces, **kwargs)


class MainMeta(ormar.ModelMeta):
    metadata = config.metadata
    database = config.database



import socket
import sqlalchemy
import databases


class BaseConfig:
    name = "App Name"
    version = "0.0.1"
    envname = "base"
    hostname = socket.gethostname()
    port = 5000
    debug = False
    envname = "prod"
    metadata = sqlalchemy.MetaData()
    dburl = "sqlite:///db.sqlite"
    database = None
    engine = None
    secret = "SECRET"
    cookie_secure = True

    def __init__(self):
        self.database = databases.Database(self.dburl)
        self.engine = sqlalchemy.create_engine(self.dburl)

    def __repr__(self):
        return (
            f"<{self.envname} config: {self.name}({self.version}) on {self.hostname}>"
        )


class DevConfig(BaseConfig):
    envname = "dev"
    debug = True
    cookie_secure = False
    dburl = "sqlite:///db.sqlite"


class TestConfig(BaseConfig):
    envname = "test"
    debug = True
    cookie_secure = False
    dburl = "sqlite:///test.sqlite"


class ProdConfig(BaseConfig):
    envname = "prod"
    secret = "MORESECURESECRET"



class route:
    def __init__(self, app, route, models, tags=[], many=False):
        self.app = app
        self.route = route
        self.models = models
        self.tags = tags
        self.many = many

    def __call__(self, cls, *args, **kwargs):
        origFetch = getattr(cls, "fetch", None)
        origCreate = getattr(cls, "create", None)
        origEdit = getattr(cls, "edit", None)
        origDestroy = getattr(cls, "destroy", None)
        fetchSuffix = " list" if self.many else ""
        models = self.models
        mname = models.base.__name__
        modelName = mname[:-4] if mname.endswith("Base") else mname
        fetchModel = models.listing if self.many else models.response
        fetchDeco = self.app.get(
            self.route,
            summary=f"Get {modelName}{fetchSuffix}",
            response_model=fetchModel,
            tags=self.tags,
        )
        createDeco = self.app.post(
            self.route,
            summary=f"Create {modelName}",
            response_model=models.response,
            tags=self.tags,
        )
        editDeco = self.app.patch(
            self.route,
            summary=f"Edit {modelName}",
            response_model=models.response,
            tags=self.tags,
        )
        destroyDeco = self.app.delete(
            self.route,
            summary=f"Destroy {modelName}",
            response_model=models.response,
            tags=self.tags,
        )

        class Wrapped(cls):
            if callable(origFetch):
                fetch = fetchDeco(origFetch)
            if callable(origCreate):
                create = createDeco(origCreate)
            if callable(origEdit):
                edit = editDeco(origEdit)
            if callable(origDestroy):
                destroy = destroyDeco(origDestroy)

        return Wrapped




from fastapi_users import FastAPIUsers, models
from fastapi_users.db import OrmarBaseUserModel, OrmarUserDatabase

from freenit.auth import authBackends
from freenit.models.mixins import MainMeta


class User(models.BaseUser):
    pass


class UserCreate(models.BaseUserCreate):
    pass


class UserUpdate(User, models.BaseUserUpdate):
    pass


class UserDB(User, models.BaseUserDB):
    pass


class UserModel(OrmarBaseUserModel):
    class Meta(MainMeta):
        tablename = "users"


userDb = OrmarUserDatabase(UserDB, UserModel)
fastapiUsers = FastAPIUsers(userDb, authBackends, User, UserCreate, UserUpdate, UserDB)



from fastapi import FastAPI
from ..config import getConfig


config = getConfig()
api = FastAPI()
api.state.database = config.database


@api.on_event("startup")
async def startup() -> None:
    database_ = api.state.database
    if not database_.is_connected:
        await database_.connect()


@api.on_event("shutdown")
async def shutdown() -> None:
    database_ = api.state.database
    if database_.is_connected:
        await database_.disconnect()



#!/usr/bin/env python

import pathlib

from setuptools import find_packages, setup
from freenit.version import version

PROJECT_ROOT = pathlib.Path(__file__).parent
README = (PROJECT_ROOT / 'README.md').read_text()

setup(
    name='freenit',
    version=version,
    description='REST API framework based on FastAPI',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/freenit-framework/backend',
    author='Goran Mekić',
    author_email='meka@tilda.center',
    license='BSD',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP',
        'Environment :: Web Environment',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    keywords=[
        'REST',
        'openapi',
        'swagger',
        'fastapi',
    ],
    packages=find_packages(exclude=['tests*']),
    python_requires='>=3.8',
    install_requires=[
        'aiosqlite',
        'alembic',
        'databases',
        'email-validator',
        'fastapi-users',
        'ormar',
        'uvicorn',
    ],
    extras_require={
        'build': [
            'twine',
        ],
        'test': [
            'pytest-asyncio',
            'pytest-factoryboy',
        ],
    },
    include_package_data=True,
    package_data={
        "": [
            "project/*",
            "project/ansible/group_vars/*",
            "project/ansible/inventory/*",
            "project/ansible/roles/devel/*",
            "project/ansible/roles/devel/tasks/*",
            "project/ansible/roles/devel/vars/*",
            "project/bin/*",
            "project/project/*",
            "project/project/api/*",
            "project/project/models/*",
            "project/templates/*",
        ]
    },
    scripts=["bin/freenit.sh"],
)



import os
import sys

import freenit.app
from alembic import context
from freenit.migration import run_migrations_offline, run_migrations_online


sys.path.append(os.getcwd())


if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()



from fastapi import Request

from freenit.auth import cookieAuthentication
from freenit.config import getConfig

from ..models.user import UserDB, fastapiUsers
from .router import api

config = getConfig()


def on_after_register(user: UserDB, request: Request):
    print(f"User {user.id} has registered.")


def on_after_forgot_password(user: UserDB, token: str, request: Request):
    print(f"User {user.id} has forgot their password. Reset token: {token}")


def after_verification_request(user: UserDB, token: str, request: Request):
    print(f"Verification requested for user {user.id}. Verification token: {token}")


tags = ["auth"]
api.include_router(
    fastapiUsers.get_auth_router(cookieAuthentication),
    prefix="/auth",
    tags=tags,
)
api.include_router(
    fastapiUsers.get_register_router(on_after_register), prefix="/auth", tags=tags
)
api.include_router(
    fastapiUsers.get_reset_password_router(
        config.secret, after_forgot_password=on_after_forgot_password
    ),
    prefix="/auth",
    tags=tags,
)
api.include_router(
    fastapiUsers.get_verify_router(
        config.secret, after_verification_request=after_verification_request
    ),
    prefix="/auth",
    tags=tags,
)
api.include_router(fastapiUsers.get_users_router(), prefix="/users", tags=["users"])
